@@ -1,6 +1,6 @@
-if not modules then modules = { } end modules ['font-ott'] = {
+if not modules then modules = { } end modules ["font-ott"] = {
     version   = 1.001,
-    comment   = "companion to font-otf.lua (tables)",
+    comment   = "companion to font-ini.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
     license   = "see context related readme files",
@@ -8,7 +8,8 @@ if not modules then modules = { } end modules ['font-ott'] = {
 }
 
 local type, next, tonumber, tostring, rawget, rawset = type, next, tonumber, tostring, rawget, rawset
-local gsub, lower, format, match = string.gsub, string.lower, string.format, string.match
+local gsub, lower, format, match, gmatch, find = string.gsub, string.lower, string.format, string.match, string.gmatch, string.find
+local sequenced = table.sequenced
 local is_boolean = string.is_boolean
 
 local setmetatableindex    = table.setmetatableindex
@@ -26,118 +27,118 @@ local statistics           = otf.statistics or { }
 otf.statistics             = statistics
 
 local scripts = allocate {
-    ['arab'] = 'arabic',
-    ['armi'] = 'imperial aramaic',
-    ['armn'] = 'armenian',
-    ['avst'] = 'avestan',
-    ['bali'] = 'balinese',
-    ['bamu'] = 'bamum',
-    ['batk'] = 'batak',
-    ['beng'] = 'bengali',
-    ['bng2'] = 'bengali variant 2',
-    ['bopo'] = 'bopomofo',
-    ['brah'] = 'brahmi',
-    ['brai'] = 'braille',
-    ['bugi'] = 'buginese',
-    ['buhd'] = 'buhid',
-    ['byzm'] = 'byzantine music',
-    ['cakm'] = 'chakma',
-    ['cans'] = 'canadian syllabics',
-    ['cari'] = 'carian',
-    ['cham'] = 'cham',
-    ['cher'] = 'cherokee',
-    ['copt'] = 'coptic',
-    ['cprt'] = 'cypriot syllabary',
-    ['cyrl'] = 'cyrillic',
-    ['deva'] = 'devanagari',
-    ['dev2'] = 'devanagari variant 2',
-    ['dsrt'] = 'deseret',
-    ['egyp'] = 'egyptian heiroglyphs',
-    ['ethi'] = 'ethiopic',
-    ['geor'] = 'georgian',
-    ['glag'] = 'glagolitic',
-    ['goth'] = 'gothic',
-    ['grek'] = 'greek',
-    ['gujr'] = 'gujarati',
-    ['gjr2'] = 'gujarati variant 2',
-    ['guru'] = 'gurmukhi',
-    ['gur2'] = 'gurmukhi variant 2',
-    ['hang'] = 'hangul',
-    ['hani'] = 'cjk ideographic',
-    ['hano'] = 'hanunoo',
-    ['hebr'] = 'hebrew',
-    ['ital'] = 'old italic',
-    ['jamo'] = 'hangul jamo',
-    ['java'] = 'javanese',
-    ['kali'] = 'kayah li',
-    ['kana'] = 'hiragana and katakana',
-    ['khar'] = 'kharosthi',
-    ['khmr'] = 'khmer',
-    ['knda'] = 'kannada',
-    ['knd2'] = 'kannada variant 2',
-    ['kthi'] = 'kaithi',
-    ['lana'] = 'tai tham',
-    ['lao' ] = 'lao',
-    ['latn'] = 'latin',
-    ['lepc'] = 'lepcha',
-    ['limb'] = 'limbu',
-    ['linb'] = 'linear b',
-    ['lisu'] = 'lisu',
-    ['lyci'] = 'lycian',
-    ['lydi'] = 'lydian',
-    ['mand'] = 'mandaic and mandaean',
-    ['math'] = 'mathematical alphanumeric symbols',
-    ['merc'] = 'meroitic cursive',
-    ['mero'] = 'meroitic hieroglyphs',
-    ['mlym'] = 'malayalam',
-    ['mlm2'] = 'malayalam variant 2',
-    ['mong'] = 'mongolian',
-    ['mtei'] = 'meitei Mayek',
-    ['musc'] = 'musical symbols',
-    ['mym2'] = 'myanmar variant 2',
-    ['mymr'] = 'myanmar',
-    ['nko' ] = "n'ko",
-    ['ogam'] = 'ogham',
-    ['olck'] = 'ol chiki',
-    ['orkh'] = 'old turkic and orkhon runic',
-    ['orya'] = 'oriya',
-    ['ory2'] = 'odia variant 2',
-    ['osma'] = 'osmanya',
-    ['phag'] = 'phags-pa',
-    ['phli'] = 'inscriptional pahlavi',
-    ['phnx'] = 'phoenician',
-    ['prti'] = 'inscriptional parthian',
-    ['rjng'] = 'rejang',
-    ['runr'] = 'runic',
-    ['samr'] = 'samaritan',
-    ['sarb'] = 'old south arabian',
-    ['saur'] = 'saurashtra',
-    ['shaw'] = 'shavian',
-    ['shrd'] = 'sharada',
-    ['sinh'] = 'sinhala',
-    ['sora'] = 'sora sompeng',
-    ['sund'] = 'sundanese',
-    ['sylo'] = 'syloti nagri',
-    ['syrc'] = 'syriac',
-    ['tagb'] = 'tagbanwa',
-    ['takr'] = 'takri',
-    ['tale'] = 'tai le',
-    ['talu'] = 'tai lu',
-    ['taml'] = 'tamil',
-    ['tavt'] = 'tai viet',
-    ['telu'] = 'telugu',
-    ['tel2'] = 'telugu variant 2',
-    ['tfng'] = 'tifinagh',
-    ['tglg'] = 'tagalog',
-    ['thaa'] = 'thaana',
-    ['thai'] = 'thai',
-    ['tibt'] = 'tibetan',
-    ['tml2'] = 'tamil variant 2',
-    ['ugar'] = 'ugaritic cuneiform',
-    ['vai' ] = 'vai',
-    ['xpeo'] = 'old persian cuneiform',
-    ['xsux'] = 'sumero-akkadian cuneiform',
-    ['yi'  ] = 'yi',
+    ["arab"] = "arabic",
+    ["armi"] = "imperial aramaic",
+    ["armn"] = "armenian",
+    ["avst"] = "avestan",
+    ["bali"] = "balinese",
+    ["bamu"] = "bamum",
+    ["batk"] = "batak",
+    ["beng"] = "bengali",
+    ["bng2"] = "bengali variant 2",
+    ["bopo"] = "bopomofo",
+    ["brah"] = "brahmi",
+    ["brai"] = "braille",
+    ["bugi"] = "buginese",
+    ["buhd"] = "buhid",
+    ["byzm"] = "byzantine music",
+    ["cakm"] = "chakma",
+    ["cans"] = "canadian syllabics",
+    ["cari"] = "carian",
+    ["cham"] = "cham",
+    ["cher"] = "cherokee",
+    ["copt"] = "coptic",
+    ["cprt"] = "cypriot syllabary",
+    ["cyrl"] = "cyrillic",
+    ["deva"] = "devanagari",
+    ["dev2"] = "devanagari variant 2",
+    ["dsrt"] = "deseret",
+    ["egyp"] = "egyptian heiroglyphs",
+    ["ethi"] = "ethiopic",
+    ["geor"] = "georgian",
+    ["glag"] = "glagolitic",
+    ["goth"] = "gothic",
+    ["grek"] = "greek",
+    ["gujr"] = "gujarati",
+    ["gjr2"] = "gujarati variant 2",
+    ["guru"] = "gurmukhi",
+    ["gur2"] = "gurmukhi variant 2",
+    ["hang"] = "hangul",
+    ["hani"] = "cjk ideographic",
+    ["hano"] = "hanunoo",
+    ["hebr"] = "hebrew",
+    ["ital"] = "old italic",
+    ["jamo"] = "hangul jamo",
+    ["java"] = "javanese",
+    ["kali"] = "kayah li",
+    ["kana"] = "hiragana and katakana",
+    ["khar"] = "kharosthi",
+    ["khmr"] = "khmer",
+    ["knda"] = "kannada",
+    ["knd2"] = "kannada variant 2",
+    ["kthi"] = "kaithi",
+    ["lana"] = "tai tham",
+    ["lao" ] = "lao",
+    ["latn"] = "latin",
+    ["lepc"] = "lepcha",
+    ["limb"] = "limbu",
+    ["linb"] = "linear b",
+    ["lisu"] = "lisu",
+    ["lyci"] = "lycian",
+    ["lydi"] = "lydian",
+    ["mand"] = "mandaic and mandaean",
+    ["math"] = "mathematical alphanumeric symbols",
+    ["merc"] = "meroitic cursive",
+    ["mero"] = "meroitic hieroglyphs",
+    ["mlym"] = "malayalam",
+    ["mlm2"] = "malayalam variant 2",
+    ["mong"] = "mongolian",
+    ["mtei"] = "meitei Mayek",
+    ["musc"] = "musical symbols",
+    ["mym2"] = "myanmar variant 2",
+    ["mymr"] = "myanmar",
+    ["nko" ] = 'n"ko',
+    ["ogam"] = "ogham",
+    ["olck"] = "ol chiki",
+    ["orkh"] = "old turkic and orkhon runic",
+    ["orya"] = "oriya",
+    ["ory2"] = "odia variant 2",
+    ["osma"] = "osmanya",
+    ["phag"] = "phags-pa",
+    ["phli"] = "inscriptional pahlavi",
+    ["phnx"] = "phoenician",
+    ["prti"] = "inscriptional parthian",
+    ["rjng"] = "rejang",
+    ["runr"] = "runic",
+    ["samr"] = "samaritan",
+    ["sarb"] = "old south arabian",
+    ["saur"] = "saurashtra",
+    ["shaw"] = "shavian",
+    ["shrd"] = "sharada",
+    ["sinh"] = "sinhala",
+    ["sora"] = "sora sompeng",
+    ["sund"] = "sundanese",
+    ["sylo"] = "syloti nagri",
+    ["syrc"] = "syriac",
+    ["tagb"] = "tagbanwa",
+    ["takr"] = "takri",
+    ["tale"] = "tai le",
+    ["talu"] = "tai lu",
+    ["taml"] = "tamil",
+    ["tavt"] = "tai viet",
+    ["telu"] = "telugu",
+    ["tel2"] = "telugu variant 2",
+    ["tfng"] = "tifinagh",
+    ["tglg"] = "tagalog",
+    ["thaa"] = "thaana",
+    ["thai"] = "thai",
+    ["tibt"] = "tibetan",
+    ["tml2"] = "tamil variant 2",
+    ["ugar"] = "ugaritic cuneiform",
+    ["vai" ] = "vai",
+    ["xpeo"] = "old persian cuneiform",
+    ["xsux"] = "sumero-akkadian cuneiform",
+    ["yi"  ] = "yi",
 }
 
 local languages = allocate {
@@ -302,7 +303,7 @@ local languages = allocate {
     ["gez" ] = "ge'ez",
     ["gih" ] = "githabul",
     ["gil" ] = "gilyak",
-    ["gil0"] = " kiribati (gilbertese)",
+    ["gil0"] = "kiribati (gilbertese)",
     ["gkp" ] = "kpelle (guinea)",
     ["glk" ] = "gilaki",
     ["gmz" ] = "gumuz",
@@ -389,6 +390,7 @@ local languages = allocate {
     ["kiu" ] = "kirmanjki",
     ["kjd" ] = "southern kiwai",
     ["kjp" ] = "eastern pwo karen",
+    ["kjz" ] = "bumthangkha",
     ["kkn" ] = "kokni",
     ["klm" ] = "kalmyk",
     ["kmb" ] = "kamba",
@@ -477,6 +479,7 @@ local languages = allocate {
     ["mdr" ] = "mandar",
     ["men" ] = "me'en",
     ["mer" ] = "meru",
+    ["mfa" ] = "pattani malay",
     ["mfe" ] = "morisyen",
     ["min" ] = "minangkabau",
     ["miz" ] = "mizo",
@@ -674,6 +677,7 @@ local languages = allocate {
     ["tpi" ] = "tok pisin",
     ["trk" ] = "turkish",
     ["tsg" ] = "tsonga",
+    ["tsj" ] = "tshangla",
     ["tua" ] = "turoyo aramaic",
     ["tul" ] = "tulu",
     ["tuv" ] = "tuvin",
@@ -704,6 +708,7 @@ local languages = allocate {
     ["xbd" ] = "lü",
     ["xhs" ] = "xhosa",
     ["xjb" ] = "minjangbal",
+    ["xkf" ] = "khengkha",
     ["xog" ] = "soga",
     ["xpe" ] = "kpelle (liberia)",
     ["yak" ] = "sakha",
@@ -726,165 +731,166 @@ local languages = allocate {
 }
 
 local features = allocate {
-    ['aalt'] = 'access all alternates',
-    ['abvf'] = 'above-base forms',
-    ['abvm'] = 'above-base mark positioning',
-    ['abvs'] = 'above-base substitutions',
-    ['afrc'] = 'alternative fractions',
-    ['akhn'] = 'akhands',
-    ['blwf'] = 'below-base forms',
-    ['blwm'] = 'below-base mark positioning',
-    ['blws'] = 'below-base substitutions',
-    ['c2pc'] = 'petite capitals from capitals',
-    ['c2sc'] = 'small capitals from capitals',
-    ['calt'] = 'contextual alternates',
-    ['case'] = 'case-sensitive forms',
-    ['ccmp'] = 'glyph composition/decomposition',
-    ['cfar'] = 'conjunct form after ro',
-    ['cjct'] = 'conjunct forms',
-    ['clig'] = 'contextual ligatures',
-    ['cpct'] = 'centered cjk punctuation',
-    ['cpsp'] = 'capital spacing',
-    ['cswh'] = 'contextual swash',
-    ['curs'] = 'cursive positioning',
-    ['dflt'] = 'default processing',
-    ['dist'] = 'distances',
-    ['dlig'] = 'discretionary ligatures',
-    ['dnom'] = 'denominators',
-    ['dtls'] = 'dotless forms', -- math
-    ['expt'] = 'expert forms',
-    ['falt'] = 'final glyph alternates',
-    ['fin2'] = 'terminal forms #2',
-    ['fin3'] = 'terminal forms #3',
-    ['fina'] = 'terminal forms',
-    ['flac'] = 'flattened accents over capitals', -- math
-    ['frac'] = 'fractions',
-    ['fwid'] = 'full width',
-    ['half'] = 'half forms',
-    ['haln'] = 'halant forms',
-    ['halt'] = 'alternate half width',
-    ['hist'] = 'historical forms',
-    ['hkna'] = 'horizontal kana alternates',
-    ['hlig'] = 'historical ligatures',
-    ['hngl'] = 'hangul',
-    ['hojo'] = 'hojo kanji forms',
-    ['hwid'] = 'half width',
-    ['init'] = 'initial forms',
-    ['isol'] = 'isolated forms',
-    ['ital'] = 'italics',
-    ['jalt'] = 'justification alternatives',
-    ['jp04'] = 'jis2004 forms',
-    ['jp78'] = 'jis78 forms',
-    ['jp83'] = 'jis83 forms',
-    ['jp90'] = 'jis90 forms',
-    ['kern'] = 'kerning',
-    ['lfbd'] = 'left bounds',
-    ['liga'] = 'standard ligatures',
-    ['ljmo'] = 'leading jamo forms',
-    ['lnum'] = 'lining figures',
-    ['locl'] = 'localized forms',
-    ['ltra'] = 'left-to-right alternates',
-    ['ltrm'] = 'left-to-right mirrored forms',
-    ['mark'] = 'mark positioning',
-    ['med2'] = 'medial forms #2',
-    ['medi'] = 'medial forms',
-    ['mgrk'] = 'mathematical greek',
-    ['mkmk'] = 'mark to mark positioning',
-    ['mset'] = 'mark positioning via substitution',
-    ['nalt'] = 'alternate annotation forms',
-    ['nlck'] = 'nlc kanji forms',
-    ['nukt'] = 'nukta forms',
-    ['numr'] = 'numerators',
-    ['onum'] = 'old style figures',
-    ['opbd'] = 'optical bounds',
-    ['ordn'] = 'ordinals',
-    ['ornm'] = 'ornaments',
-    ['palt'] = 'proportional alternate width',
-    ['pcap'] = 'petite capitals',
-    ['pkna'] = 'proportional kana',
-    ['pnum'] = 'proportional figures',
-    ['pref'] = 'pre-base forms',
-    ['pres'] = 'pre-base substitutions',
-    ['pstf'] = 'post-base forms',
-    ['psts'] = 'post-base substitutions',
-    ['pwid'] = 'proportional widths',
-    ['qwid'] = 'quarter widths',
-    ['rand'] = 'randomize',
-    ['rclt'] = 'required contextual alternates',
-    ['rkrf'] = 'rakar forms',
-    ['rlig'] = 'required ligatures',
-    ['rphf'] = 'reph form',
-    ['rtbd'] = 'right bounds',
-    ['rtla'] = 'right-to-left alternates',
-    ['rtlm'] = 'right to left math', -- math
-    ['ruby'] = 'ruby notation forms',
-    ['salt'] = 'stylistic alternates',
-    ['sinf'] = 'scientific inferiors',
-    ['size'] = 'optical size',
-    ['smcp'] = 'small capitals',
-    ['smpl'] = 'simplified forms',
- -- ['ss01'] = 'stylistic set 1',
- -- ['ss02'] = 'stylistic set 2',
- -- ['ss03'] = 'stylistic set 3',
- -- ['ss04'] = 'stylistic set 4',
- -- ['ss05'] = 'stylistic set 5',
- -- ['ss06'] = 'stylistic set 6',
- -- ['ss07'] = 'stylistic set 7',
- -- ['ss08'] = 'stylistic set 8',
- -- ['ss09'] = 'stylistic set 9',
- -- ['ss10'] = 'stylistic set 10',
- -- ['ss11'] = 'stylistic set 11',
- -- ['ss12'] = 'stylistic set 12',
- -- ['ss13'] = 'stylistic set 13',
- -- ['ss14'] = 'stylistic set 14',
- -- ['ss15'] = 'stylistic set 15',
- -- ['ss16'] = 'stylistic set 16',
- -- ['ss17'] = 'stylistic set 17',
- -- ['ss18'] = 'stylistic set 18',
- -- ['ss19'] = 'stylistic set 19',
- -- ['ss20'] = 'stylistic set 20',
-    ['ssty'] = 'script style', -- math
-    ['stch'] = 'stretching glyph decomposition',
-    ['subs'] = 'subscript',
-    ['sups'] = 'superscript',
-    ['swsh'] = 'swash',
-    ['titl'] = 'titling',
-    ['tjmo'] = 'trailing jamo forms',
-    ['tnam'] = 'traditional name forms',
-    ['tnum'] = 'tabular figures',
-    ['trad'] = 'traditional forms',
-    ['twid'] = 'third widths',
-    ['unic'] = 'unicase',
-    ['valt'] = 'alternate vertical metrics',
-    ['vatu'] = 'vattu variants',
-    ['vert'] = 'vertical writing',
-    ['vhal'] = 'alternate vertical half metrics',
-    ['vjmo'] = 'vowel jamo forms',
-    ['vkna'] = 'vertical kana alternates',
-    ['vkrn'] = 'vertical kerning',
-    ['vpal'] = 'proportional alternate vertical metrics',
-    ['vrt2'] = 'vertical rotation',
-    ['zero'] = 'slashed zero',
+    ["aalt"] = "access all alternates",
+    ["abvf"] = "above-base forms",
+    ["abvm"] = "above-base mark positioning",
+    ["abvs"] = "above-base substitutions",
+    ["afrc"] = "alternative fractions",
+    ["akhn"] = "akhands",
+    ["blwf"] = "below-base forms",
+    ["blwm"] = "below-base mark positioning",
+    ["blws"] = "below-base substitutions",
+    ["c2pc"] = "petite capitals from capitals",
+    ["c2sc"] = "small capitals from capitals",
+    ["calt"] = "contextual alternates",
+    ["case"] = "case-sensitive forms",
+    ["ccmp"] = "glyph composition/decomposition",
+    ["cfar"] = "conjunct form after ro",
+    ["cjct"] = "conjunct forms",
+    ["clig"] = "contextual ligatures",
+    ["cpct"] = "centered cjk punctuation",
+    ["cpsp"] = "capital spacing",
+    ["cswh"] = "contextual swash",
+    ["curs"] = "cursive positioning",
+    ["dflt"] = "default processing",
+    ["dist"] = "distances",
+    ["dlig"] = "discretionary ligatures",
+    ["dnom"] = "denominators",
+    ["dtls"] = "dotless forms", -- math
+    ["expt"] = "expert forms",
+    ["falt"] = "final glyph alternates",
+    ["fin2"] = "terminal forms #2",
+    ["fin3"] = "terminal forms #3",
+    ["fina"] = "terminal forms",
+    ["flac"] = "flattened accents over capitals", -- math
+    ["frac"] = "fractions",
+    ["fwid"] = "full width",
+    ["half"] = "half forms",
+    ["haln"] = "halant forms",
+    ["halt"] = "alternate half width",
+    ["hist"] = "historical forms",
+    ["hkna"] = "horizontal kana alternates",
+    ["hlig"] = "historical ligatures",
+    ["hngl"] = "hangul",
+    ["hojo"] = "hojo kanji forms",
+    ["hwid"] = "half width",
+    ["init"] = "initial forms",
+    ["isol"] = "isolated forms",
+    ["ital"] = "italics",
+    ["jalt"] = "justification alternatives",
+    ["jp04"] = "jis2004 forms",
+    ["jp78"] = "jis78 forms",
+    ["jp83"] = "jis83 forms",
+    ["jp90"] = "jis90 forms",
+    ["kern"] = "kerning",
+    ["lfbd"] = "left bounds",
+    ["liga"] = "standard ligatures",
+    ["ljmo"] = "leading jamo forms",
+    ["lnum"] = "lining figures",
+    ["locl"] = "localized forms",
+    ["ltra"] = "left-to-right alternates",
+    ["ltrm"] = "left-to-right mirrored forms",
+    ["mark"] = "mark positioning",
+    ["med2"] = "medial forms #2",
+    ["medi"] = "medial forms",
+    ["mgrk"] = "mathematical greek",
+    ["mkmk"] = "mark to mark positioning",
+    ["mset"] = "mark positioning via substitution",
+    ["nalt"] = "alternate annotation forms",
+    ["nlck"] = "nlc kanji forms",
+    ["nukt"] = "nukta forms",
+    ["numr"] = "numerators",
+    ["onum"] = "old style figures",
+    ["opbd"] = "optical bounds",
+    ["ordn"] = "ordinals",
+    ["ornm"] = "ornaments",
+    ["palt"] = "proportional alternate width",
+    ["pcap"] = "petite capitals",
+    ["pkna"] = "proportional kana",
+    ["pnum"] = "proportional figures",
+    ["pref"] = "pre-base forms",
+    ["pres"] = "pre-base substitutions",
+    ["pstf"] = "post-base forms",
+    ["psts"] = "post-base substitutions",
+    ["pwid"] = "proportional widths",
+    ["qwid"] = "quarter widths",
+    ["rand"] = "randomize",
+    ["rclt"] = "required contextual alternates",
+    ["rkrf"] = "rakar forms",
+    ["rlig"] = "required ligatures",
+    ["rphf"] = "reph form",
+    ["rtbd"] = "right bounds",
+    ["rtla"] = "right-to-left alternates",
+    ["rtlm"] = "right to left mirrored forms",
+    ["rvrn"] = "required variation alternates",
+    ["ruby"] = "ruby notation forms",
+    ["salt"] = "stylistic alternates",
+    ["sinf"] = "scientific inferiors",
+    ["size"] = "optical size", -- now stat table
+    ["smcp"] = "small capitals",
+    ["smpl"] = "simplified forms",
+ -- ["ss01"] = "stylistic set 1",
+ -- ["ss02"] = "stylistic set 2",
+ -- ["ss03"] = "stylistic set 3",
+ -- ["ss04"] = "stylistic set 4",
+ -- ["ss05"] = "stylistic set 5",
+ -- ["ss06"] = "stylistic set 6",
+ -- ["ss07"] = "stylistic set 7",
+ -- ["ss08"] = "stylistic set 8",
+ -- ["ss09"] = "stylistic set 9",
+ -- ["ss10"] = "stylistic set 10",
+ -- ["ss11"] = "stylistic set 11",
+ -- ["ss12"] = "stylistic set 12",
+ -- ["ss13"] = "stylistic set 13",
+ -- ["ss14"] = "stylistic set 14",
+ -- ["ss15"] = "stylistic set 15",
+ -- ["ss16"] = "stylistic set 16",
+ -- ["ss17"] = "stylistic set 17",
+ -- ["ss18"] = "stylistic set 18",
+ -- ["ss19"] = "stylistic set 19",
+ -- ["ss20"] = "stylistic set 20",
+    ["ssty"] = "script style", -- math
+    ["stch"] = "stretching glyph decomposition",
+    ["subs"] = "subscript",
+    ["sups"] = "superscript",
+    ["swsh"] = "swash",
+    ["titl"] = "titling",
+    ["tjmo"] = "trailing jamo forms",
+    ["tnam"] = "traditional name forms",
+    ["tnum"] = "tabular figures",
+    ["trad"] = "traditional forms",
+    ["twid"] = "third widths",
+    ["unic"] = "unicase",
+    ["valt"] = "alternate vertical metrics",
+    ["vatu"] = "vattu variants",
+    ["vert"] = "vertical writing",
+    ["vhal"] = "alternate vertical half metrics",
+    ["vjmo"] = "vowel jamo forms",
+    ["vkna"] = "vertical kana alternates",
+    ["vkrn"] = "vertical kerning",
+    ["vpal"] = "proportional alternate vertical metrics",
+    ["vrt2"] = "vertical rotation",
+    ["zero"] = "slashed zero",
 
-    ['trep'] = 'traditional tex replacements',
-    ['tlig'] = 'traditional tex ligatures',
+    ["trep"] = "traditional tex replacements",
+    ["tlig"] = "traditional tex ligatures",
 
-    ['ss..'] = 'stylistic set ..',
-    ['cv..'] = 'character variant ..',
-    ['js..'] = 'justification ..',
+    ["ss.."] = "stylistic set ..",
+    ["cv.."] = "character variant ..",
+    ["js.."] = "justification ..",
 
     ["dv.."] = "devanagari ..",
     ["ml.."] = "malayalam ..",
 }
 
 local baselines = allocate {
-    ['hang'] = 'hanging baseline',
-    ['icfb'] = 'ideographic character face bottom edge baseline',
-    ['icft'] = 'ideographic character face tope edige baseline',
-    ['ideo'] = 'ideographic em-box bottom edge baseline',
-    ['idtp'] = 'ideographic em-box top edge baseline',
-    ['math'] = 'mathematical centered baseline',
-    ['romn'] = 'roman baseline'
+    ["hang"] = "hanging baseline",
+    ["icfb"] = "ideographic character face bottom edge baseline",
+    ["icft"] = "ideographic character face tope edige baseline",
+    ["ideo"] = "ideographic em-box bottom edge baseline",
+    ["idtp"] = "ideographic em-box top edge baseline",
+    ["math"] = "mathematical centered baseline",
+    ["romn"] = "roman baseline"
 }
 
 tables.scripts   = scripts
@@ -986,9 +992,13 @@ setmetatableindex(languages, function(t,k)
     return "dflt"
 end)
 
-setmetatablenewindex(languages, "ignore")
-setmetatablenewindex(baselines, "ignore")
-setmetatablenewindex(baselines, "ignore")
+if setmetatablenewindex then
+
+    setmetatablenewindex(languages, "ignore")
+    setmetatablenewindex(scripts,   "ignore")
+    setmetatablenewindex(baselines, "ignore")
+
+end
 
 local function resolve(t,k)
     if k then
@@ -1028,7 +1038,11 @@ local function assign(t,k,v)
     end
 end
 
-setmetatablenewindex(features, assign)
+if setmetatablenewindex then
+
+    setmetatablenewindex(features, assign)
+
+end
 
 local checkers = {
     rand = function(v)
@@ -1084,7 +1098,7 @@ storage.register("fonts/otf/usedfeatures", usedfeatures, "fonts.handlers.otf.sta
 
 local normalizedaxis = otf.readers.helpers.normalizedaxis or function(s) return s end
 
-function otffeatures.normalize(features)
+function otffeatures.normalize(features,wrap) -- wrap is for context
     if features then
         local h = { }
         for key, value in next, features do
@@ -1097,9 +1111,9 @@ function otffeatures.normalize(features)
                 h.script = rawget(verbosescripts,v) or (scripts[v] and v) or "dflt" -- auto adds
             elseif k == "axis" then
                 h[k] = normalizedaxis(value)
-if not callbacks.supported.glyph_stream_provider then
-    h.variableshapes = true -- for the moment
-end
+                if not callbacks.supported.glyph_stream_provider then
+                    h.variableshapes = true -- for the moment
+                end
             else
                 local uk = usedfeatures[key]
                 local uv = uk[value]
@@ -1112,10 +1126,27 @@ end
                     elseif type(value) == "string" then
                         local b = is_boolean(value)
                         if type(b) == "nil" then
-                            uv = lower(value)
+                         -- we do this elsewhere
+                         --
+                         -- if find(value,"=") then
+                         --     local t = { }
+                         --     for k, v in gmatch(value,"([^%s,=]+)%s*=%s*([^%s,=]+)") do
+                         --         t[k] = tonumber(v) or v
+                         --     end
+                         --     if next(t) then
+                         --         value = sequenced(t,",")
+                         --     end
+                         -- end
+                            if wrap and find(value,",") then
+                                uv = "{"..lower(value).."}"
+                            else
+                                uv = lower(value)
+                            end
                         else
                             uv = b
                         end
+                    elseif type(value) == "table" then
+                        uv = sequenced(t,",")
                     else
                         uv = value
                     end
