@@ -6,12 +6,11 @@ if not modules then modules = { } end modules ['l-table'] = {
     license   = "see context related readme files"
 }
 
-local type, next, tostring, tonumber, ipairs, select = type, next, tostring, tonumber, ipairs, select
+local type, next, tostring, tonumber, select = type, next, tostring, tonumber, select
 local table, string = table, string
-local concat, sort, insert, remove = table.concat, table.sort, table.insert, table.remove
+local concat, sort = table.concat, table.sort
 local format, lower, dump = string.format, string.lower, string.dump
 local getmetatable, setmetatable = getmetatable, setmetatable
-local getinfo = debug.getinfo
 local lpegmatch, patterns = lpeg.match, lpeg.patterns
 local floor = math.floor
 
@@ -22,8 +21,13 @@ local floor = math.floor
 
 local stripper = patterns.stripper
 
+function table.getn(t)
+    return t and #t -- for very old times sake
+end
+
 function table.strip(tab)
-    local lst, l = { }, 0
+    local lst = { }
+    local l   = 0
     for i=1,#tab do
         local s = lpegmatch(stripper,tab[i]) or ""
         if s == "" then
@@ -38,7 +42,8 @@ end
 
 function table.keys(t)
     if t then
-        local keys, k = { }, 0
+        local keys = { }
+        local k    = 0
         for key in next, t do
             k = k + 1
             keys[k] = key
@@ -142,28 +147,30 @@ end
 
 local function sortedkeys(tab)
     if tab then
-        local srt, category, s = { }, 0, 0 -- 0=unknown 1=string, 2=number 3=mixed
+        local srt      = { }
+        local category = 0 -- 0=unknown 1=string, 2=number 3=mixed
+        local s        = 0
         for key in next, tab do
             s = s + 1
             srt[s] = key
-            if category == 3 then
-                -- no further check
-            elseif category == 1 then
-                if type(key) ~= "string" then
-                    category = 3
-                end
-            elseif category == 2 then
-                if type(key) ~= "number" then
-                    category = 3
-                end
-            else
+            if category ~= 3 then
                 local tkey = type(key)
-                if tkey == "string" then
-                    category = 1
-                elseif tkey == "number" then
-                    category = 2
+                if category == 1 then
+                    if tkey ~= "string" then
+                        category = 3
+                    end
+                elseif category == 2 then
+                    if tkey ~= "number" then
+                        category = 3
+                    end
                 else
-                    category = 3
+                    if tkey == "string" then
+                        category = 1
+                    elseif tkey == "number" then
+                        category = 2
+                    else
+                        category = 3
+                    end
                 end
             end
         end
@@ -182,7 +189,8 @@ end
 
 local function sortedhashonly(tab)
     if tab then
-        local srt, s = { }, 0
+        local srt = { }
+        local s   = 0
         for key in next, tab do
             if type(key) == "string" then
                 s = s + 1
@@ -200,7 +208,8 @@ end
 
 local function sortedindexonly(tab)
     if tab then
-        local srt, s = { }, 0
+        local srt = { }
+        local s   = 0
         for key in next, tab do
             if type(key) == "number" then
                 s = s + 1
@@ -218,7 +227,8 @@ end
 
 local function sortedhashkeys(tab,cmp) -- fast one
     if tab then
-        local srt, s = { }, 0
+        local srt = { }
+        local s   = 0
         for key in next, tab do
             if key then
                 s= s + 1
@@ -314,7 +324,9 @@ end
 -- end
 
 function table.merge(t, ...) -- first one is target
-    t = t or { }
+    if not t then
+        t = { }
+    end
     for i=1,select("#",...) do
         for k, v in next, (select(i,...)) do
             t[k] = v
@@ -380,7 +392,8 @@ end
 -- end
 
 function table.imerged(...)
-    local tmp, ntmp = { }, 0
+    local tmp  = { }
+    local ntmp = 0
     for i=1,select("#",...) do
         local nst = select(i,...)
         for j=1,#nst do
@@ -416,8 +429,10 @@ end
 
 -- todo : copy without metatable
 
-local function copy(t, tables) -- taken from lua wiki, slightly adapted
-    tables = tables or { }
+local function copy(t,tables) -- taken from lua wiki, slightly adapted
+    if not tables then
+        tables = { }
+    end
     local tcopy = { }
     if not tables[t] then
         tables[t] = tcopy
@@ -427,7 +442,7 @@ local function copy(t, tables) -- taken from lua wiki, slightly adapted
             if tables[i] then
                 i = tables[i]
             else
-                i = copy(i, tables)
+                i = copy(i,tables)
             end
         end
         if type(v) ~= "table" then
@@ -435,7 +450,7 @@ local function copy(t, tables) -- taken from lua wiki, slightly adapted
         elseif tables[v] then
             tcopy[i] = tables[v]
         else
-            tcopy[i] = copy(v, tables)
+            tcopy[i] = copy(v,tables)
         end
     end
     local mt = getmetatable(t)
@@ -460,7 +475,7 @@ function table.tohash(t,value)
     local h = { }
     if t then
         if value == nil then value = true end
-        for _, v in next, t do -- no ipairs here
+        for _, v in next, t do
             h[v] = value
         end
     end
@@ -468,8 +483,9 @@ function table.tohash(t,value)
 end
 
 function table.fromhash(t)
-    local hsh, h = { }, 0
-    for k, v in next, t do -- no ipairs here
+    local hsh = { }
+    local h   = 0
+    for k, v in next, t do
         if v then
             h = h + 1
             hsh[h] = k
@@ -666,7 +682,8 @@ local function do_serialize(root,name,depth,level,indexed)
     end
     -- we could check for k (index) being number (cardinal)
     if root and next(root) ~= nil then
-        local first, last = nil, 0
+        local first = nil
+        local last  = 0
         if compact then
             last = #root
             for k=1,last do
@@ -826,22 +843,25 @@ local function do_serialize(root,name,depth,level,indexed)
                 end
             elseif tv == "function" then
                 if functions then
-                    local f = getinfo(v).what == "C" and dump(dummy) or dump(v) -- maybe strip
-                 -- local f = getinfo(v).what == "C" and dump(function(...) return v(...) end) or dump(v) -- maybe strip
-                    if tk == "number" then
-                        if hexify then
-                            handle(format("%s [0x%X]=load(%q),",depth,k,f))
+                    local getinfo = debug and debug.getinfo
+                    if getinfo then
+                        local f = getinfo(v).what == "C" and dump(dummy) or dump(v) -- maybe strip
+                     -- local f = getinfo(v).what == "C" and dump(function(...) return v(...) end) or dump(v) -- maybe strip
+                        if tk == "number" then
+                            if hexify then
+                                handle(format("%s [0x%X]=load(%q),",depth,k,f))
+                            else
+                                handle(format("%s [%s]=load(%q),",depth,k,f))
+                            end
+                        elseif tk == "boolean" then
+                            handle(format("%s [%s]=load(%q),",depth,k and "true" or "false",f))
+                        elseif tk ~= "string" then
+                            -- ignore
+                        elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
+                            handle(format("%s %s=load(%q),",depth,k,f))
                         else
-                            handle(format("%s [%s]=load(%q),",depth,k,f))
+                            handle(format("%s [%q]=load(%q),",depth,k,f))
                         end
-                    elseif tk == "boolean" then
-                        handle(format("%s [%s]=load(%q),",depth,k and "true" or "false",f))
-                    elseif tk ~= "string" then
-                        -- ignore
-                    elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
-                        handle(format("%s %s=load(%q),",depth,k,f))
-                    else
-                        handle(format("%s [%q]=load(%q),",depth,k,f))
                     end
                 end
             else
@@ -954,7 +974,8 @@ end
 -- number   : [number] = { }
 
 function table.serialize(root,name,specification)
-    local t, n = { }, 0
+    local t = { }
+    local n = 0
     local function flush(s)
         n = n + 1
         t[n] = s
@@ -978,13 +999,15 @@ function table.tofile(filename,root,name,specification)
     local f = io.open(filename,'w')
     if f then
         if maxtab > 1 then
-            local t, n = { }, 0
+            local t = { }
+            local n = 0
             local function flush(s)
                 n = n + 1
                 t[n] = s
                 if n > maxtab then
                     f:write(concat(t,"\n"),"\n") -- hm, write(sometable) should be nice
-                    t, n = { }, 0 -- we could recycle t if needed
+                    t = { } -- we could recycle t if needed
+                    n = 0
                 end
             end
             serialize(flush,root,name,specification)
@@ -1002,12 +1025,12 @@ end
 
 local function flattened(t,f,depth) -- also handles { nil, 1, nil, 2 }
     if f == nil then
-        f = { }
+        f     = { }
         depth = 0xFFFF
     elseif tonumber(f) then
         -- assume that only two arguments are given
         depth = f
-        f = { }
+        f     = { }
     elseif not depth then
         depth = 0xFFFF
     end
@@ -1092,9 +1115,15 @@ function table.unnest(t) -- bad name
 end
 
 local function are_equal(a,b,n,m) -- indexed
-    if a and b and #a == #b then
-        n = n or 1
-        m = m or #a
+    if a == b then
+        return true
+    elseif a and b and #a == #b then
+        if not n then
+            n = 1
+        end
+        if not m then
+            m = #a
+        end
         for i=n,m do
             local ai, bi = a[i], b[i]
             if ai==bi then
@@ -1114,16 +1143,18 @@ local function are_equal(a,b,n,m) -- indexed
 end
 
 local function identical(a,b) -- assumes same structure
-    for ka, va in next, a do
-        local vb = b[ka]
-        if va == vb then
-            -- same
-        elseif type(va) == "table" and  type(vb) == "table" then
-            if not identical(va,vb) then
+    if a ~= b then
+        for ka, va in next, a do
+            local vb = b[ka]
+            if va == vb then
+                -- same
+            elseif type(va) == "table" and  type(vb) == "table" then
+                if not identical(va,vb) then
+                    return false
+                end
+            else
                 return false
             end
-        else
-            return false
         end
     end
     return true
@@ -1205,7 +1236,8 @@ end
 
 function table.reversed(t)
     if t then
-        local tt, tn = { }, #t
+        local tt = { }
+        local tn = #t
         if tn > 0 then
             local ttn = 0
             for i=tn,1,-1 do
@@ -1217,27 +1249,35 @@ function table.reversed(t)
     end
 end
 
-function table.reverse(t)
+function table.reverse(t) -- check with 5.3 ?
     if t then
         local n = #t
-        for i=1,floor(n/2) do
-            local j = n - i + 1
+        local m = n + 1
+        for i=1,floor(n/2) do -- maybe just n//2
+            local j = m - i
             t[i], t[j] = t[j], t[i]
         end
         return t
     end
 end
 
-function table.sequenced(t,sep,simple) -- hash only
+local function sequenced(t,sep,simple)
     if not t then
         return ""
+    elseif type(t) == "string" then
+        return t -- handy fallback
     end
     local n = #t
     local s = { }
     if n > 0 then
         -- indexed
         for i=1,n do
-            s[i] = tostring(t[i])
+            local v = t[i]
+            if type(v) == "table" then
+                s[i] = "{" .. sequenced(v,sep,simple) .. "}"
+            else
+                s[i] = tostring(t[i])
+            end
         end
     else
         -- hashed
@@ -1249,17 +1289,27 @@ function table.sequenced(t,sep,simple) -- hash only
                     s[n] = k
                 elseif v and v~= "" then
                     n = n + 1
-                    s[n] = k .. "=" .. tostring(v)
+                    if type(v) == "table" then
+                        s[n] = k .. "={" .. sequenced(v,sep,simple) .. "}"
+                    else
+                        s[n] = k .. "=" .. tostring(v)
+                    end
                 end
             else
                 n = n + 1
-                s[n] = k .. "=" .. tostring(v)
+                if type(v) == "table" then
+                    s[n] = k .. "={" .. sequenced(v,sep,simple) .. "}"
+                else
+                    s[n] = k .. "=" .. tostring(v)
+                end
             end
         end
     end
     return concat(s,sep or " | ")
 end
 
+table.sequenced = sequenced
+
 function table.print(t,...)
     if type(t) ~= "table" then
         print(tostring(t))
@@ -1304,8 +1354,8 @@ end
 
 function table.unique(old)
     local hash = { }
-    local new = { }
-    local n = 0
+    local new  = { }
+    local n    = 0
     for i=1,#old do
         local oi = old[i]
         if not hash[oi] then
@@ -1326,12 +1376,14 @@ end
 
 function table.values(t,s) -- optional sort flag
     if t then
-        local values, keys, v = { }, { }, 0
+        local values = { }
+        local keys   = { }
+        local v      = 0
         for key, value in next, t do
             if not keys[value] then
                 v = v + 1
                 values[v] = value
-                keys[k] = key
+                keys[k]   = key
             end
         end
         if s then
@@ -1388,3 +1440,26 @@ function table.filtered(t,pattern,sort,cmp)
         return nothing
     end
 end
+
+-- lua 5.3:
+
+if not table.move then
+
+    function table.move(a1,f,e,t,a2)
+        if a2 and a1 ~= a2 then
+            for i=f,e do
+                a2[t] = a1[i]
+                t = t + 1
+            end
+            return a2
+        else
+            t = t + e - f
+            for i=e,f,-1 do
+                a1[t] = a1[i]
+                t = t - 1
+            end
+            return a1
+        end
+    end
+
+end
