@@ -15,9 +15,9 @@ if not modules then modules = { } end modules ['lpdf-mis'] = {
 -- referencing and references had to be tracked in multiple passes. Of
 -- course there are a couple of more changes.
 
-local next, tostring = next, tostring
+local next, tostring, type = next, tostring, type
 local format, gsub, formatters = string.format, string.gsub, string.formatters
-local texset, texget = tex.set, tex.get
+local flattened = table.flattened
 
 local backends, lpdf, nodes = backends, lpdf, nodes
 
@@ -25,10 +25,11 @@ local nodeinjections       = backends.pdf.nodeinjections
 local codeinjections       = backends.pdf.codeinjections
 local registrations        = backends.pdf.registrations
 
-local copy_node            = node.copy
+local nuts                 = nodes.nuts
+local copy_node            = nuts.copy
 
-local nodepool             = nodes.pool
-local pdfliteral           = nodepool.pdfliteral
+local nodepool             = nuts.pool
+local pageliteral          = nodepool.pageliteral
 local register             = nodepool.register
 
 local pdfdictionary        = lpdf.dictionary
@@ -41,6 +42,7 @@ local pdfstring            = lpdf.string
 local pdfflushobject       = lpdf.flushobject
 local pdfflushstreamobject = lpdf.flushstreamobject
 local pdfaction            = lpdf.action
+local pdfminorversion      = lpdf.minorversion
 
 local formattedtimestamp   = lpdf.pdftimestamp
 local adddocumentextgstate = lpdf.adddocumentextgstate
@@ -49,6 +51,10 @@ local addtoinfo            = lpdf.addtoinfo
 local addtopageattributes  = lpdf.addtopageattributes
 local addtonames           = lpdf.addtonames
 
+local pdfgetmetadata       = lpdf.getmetadata
+
+local texset               = tex.set
+
 local variables            = interfaces.variables
 
 local v_stop               = variables.stop
@@ -65,11 +71,20 @@ local v_landscape          = variables.landscape
 local v_portrait           = variables.portrait
 local v_page               = variables.page
 local v_paper              = variables.paper
+local v_attachment         = variables.attachment
+local v_layer              = variables.layer
+local v_lefttoright        = variables.lefttoright
+local v_righttoleft        = variables.righttoleft
+local v_title              = variables.title
 
-local positive             = register(pdfliteral("/GSpositive gs"))
-local negative             = register(pdfliteral("/GSnegative gs"))
-local overprint            = register(pdfliteral("/GSoverprint gs"))
-local knockout             = register(pdfliteral("/GSknockout gs"))
+local positive             = register(pageliteral("/GSpositive gs"))
+local negative             = register(pageliteral("/GSnegative gs"))
+local overprint            = register(pageliteral("/GSoverprint gs"))
+local knockout             = register(pageliteral("/GSknockout gs"))
+
+local omitextraboxes       = false
+
+directives.register("backend.omitextraboxes", function(v) omitextraboxes = v end)
 
 local function initializenegative()
     local a = pdfarray { 0, 1 }
@@ -79,7 +94,7 @@ local function initializenegative()
         Range        = a,
         Domain       = a,
     }
-    local negative = pdfdictionary { Type = g, TR = pdfreference(pdfflushstreamobject("{ 1 exch sub }",d)) }
+    local negative = pdfdictionary { Type = g, TR = pdfreference(pdfflushstreamobject("{ 1 exch sub }",d)) } -- can be shared
     local positive = pdfdictionary { Type = g, TR = pdfconstant("Identity") }
     adddocumentextgstate("GSnegative", pdfreference(pdfflushobject(negative)))
     adddocumentextgstate("GSpositive", pdfreference(pdfflushobject(positive)))
@@ -184,6 +199,12 @@ local done = false  -- using "setupidentity = function() end" fails as the meani
 
 local function setupidentity()
     if not done then
+        local metadata = pdfgetmetadata()
+        local creator  = metadata.creator
+        local version  = metadata.contextversion
+        local time     = metadata.time
+        local jobname  = environment.jobname or tex.jobname or "unknown"
+        --
         local title = identity.title
         if not title or title == "" then
             title = tex.jobname
@@ -197,21 +218,16 @@ local function setupidentity()
         if author ~= "" then
             addtoinfo("Author",  pdfunicode(author), author) -- '/Author' in /Info, 'Creator' in XMP
         end
-     -- local creator = identity.creator or ""
-        local creator = "LuaTeX + ConTeXt MkIV" -- has to be the same in CreatorTool
-        if creator ~= "" then
-            addtoinfo("Creator", pdfunicode(creator), creator) -- '/Creator' in /Info, 'CreatorTool' in XMP
-        end
-        local currenttimestamp = lpdf.timestamp()
-        addtoinfo("CreationDate", pdfstring(formattedtimestamp(currenttimestamp)))
+        addtoinfo("Creator", pdfunicode(creator), creator)
+        addtoinfo("CreationDate", pdfstring(formattedtimestamp(time)))
         local date = identity.date or ""
-        local pdfdate = formattedtimestamp(date)
+        local pdfdate = date and formattedtimestamp(date)
         if pdfdate then
             addtoinfo("ModDate", pdfstring(pdfdate), date)
         else
             -- users should enter the date in 2010-01-19T23:27:50+01:00 format
             -- and if not provided that way we use the creation time instead
-            addtoinfo("ModDate", pdfstring(formattedtimestamp(currenttimestamp)), currenttimestamp)
+            addtoinfo("ModDate", pdfstring(formattedtimestamp(time)),time)
         end
         local keywords = identity.keywords or ""
         if keywords ~= "" then
@@ -221,11 +237,18 @@ local function setupidentity()
         local id = lpdf.id()
         addtoinfo("ID", pdfstring(id), id) -- needed for pdf/x
         --
-        addtoinfo("ConTeXt.Version", environment.version)
-        addtoinfo("ConTeXt.Time",    os.date("%Y-%m-%d %H:%M"))
-        addtoinfo("ConTeXt.Jobname", environment.jobname or tex.jobname)
-        addtoinfo("ConTeXt.Url",     "www.pragma-ade.com")
-        addtoinfo("ConTeXt.Support", "contextgarden.net")
+        addtoinfo("ConTeXt.Version",version)
+        --
+        local lmtx = codeinjections.lmtxmode()
+        if lmtx then
+            addtoinfo("ConTeXt.LMTX",formatters["%0.2f"](lmtx))
+        end
+        --
+        addtoinfo("ConTeXt.Time",os.date("%Y-%m-%d %H:%M"))
+        addtoinfo("ConTeXt.Jobname",jobname)
+        addtoinfo("ConTeXt.Url","www.pragma-ade.com")
+        addtoinfo("ConTeXt.Support","contextgarden.net")
+        addtoinfo("TeX.Support","tug.org")
         --
         done = true
     else
@@ -245,7 +268,9 @@ local function flushjavascripts()
         local a = pdfarray()
         local pdf_javascript = pdfconstant("JavaScript")
         for i=1,#t do
-            local name, script = t[i][1], t[i][2]
+            local ti     = t[i]
+            local name   = ti[1]
+            local script = ti[2]
             local j = pdfdictionary {
                 S  = pdf_javascript,
                 JS = pdfreference(pdfflushstreamobject(script)),
@@ -261,15 +286,63 @@ lpdf.registerdocumentfinalizer(flushjavascripts,"javascripts")
 
 -- -- --
 
-local pagespecs = {
-    [v_none] = {
-    },
+local plusspecs = {
     [v_max] = {
         mode = "FullScreen",
     },
     [v_bookmark] = {
         mode = "UseOutlines",
     },
+    [v_attachment] = {
+        mode = "UseAttachments",
+    },
+    [v_layer] = {
+        mode = "UseOC",
+    },
+    [v_fit] = {
+        fit  = true,
+    },
+    [v_doublesided] = {
+        layout = "TwoColumnRight",
+    },
+    [v_fixed] = {
+        fixed  = true,
+    },
+    [v_landscape] = {
+        duplex = "DuplexFlipShortEdge",
+    },
+    [v_portrait] = {
+        duplex = "DuplexFlipLongEdge",
+    },
+    [v_page] = {
+        duplex = "Simplex" ,
+    },
+    [v_paper] = {
+        paper  = true,
+    },
+    [v_title] ={
+        title = true,
+    },
+    [v_lefttoright] ={
+        direction = "L2R",
+    },
+    [v_righttoleft] ={
+        direction = "R2R",
+    },
+}
+
+local pagespecs = {
+    --
+    [v_max]         = plusspecs[v_max],
+    [v_bookmark]    = plusspecs[v_bookmark],
+    [v_attachment]  = plusspecs[v_attachment],
+    [v_layer]       = plusspecs[v_layer],
+    [v_lefttoright] = plusspecs[v_lefttoright],
+    [v_righttoleft] = plusspecs[v_righttoleft],
+    [v_title]       = plusspecs[v_title],
+    --
+    [v_none] = {
+    },
     [v_fit] = {
         mode = "UseNone",
         fit  = true,
@@ -322,55 +395,29 @@ local pagespecs = {
     },
 }
 
-local plusspecs = {
-    [v_max] = {
-        mode = "FullScreen",
-    },
-    [v_bookmark] = {
-        mode = "UseOutlines",
-    },
-    [v_fit] = {
-        fit  = true,
-    },
-    [v_doublesided] = {
-        layout = "TwoColumnRight",
-    },
-    [v_fixed] = {
-        fixed  = true,
-    },
-    [v_landscape] = {
-        duplex = "DuplexFlipShortEdge",
-    },
-    [v_portrait] = {
-        duplex = "DuplexFlipLongEdge",
-    },
-    [v_page] = {
-        duplex = "Simplex" ,
-    },
-    [v_paper] = {
-        paper  = true,
-    },
-}
-
 local pagespec, topoffset, leftoffset, height, width, doublesided = "default", 0, 0, 0, 0, false
 local cropoffset, bleedoffset, trimoffset, artoffset = 0, 0, 0, 0
+local marked = false
 local copies = false
 
+local getpagedimensions  getpagedimensions = function()
+    getpagedimensions = backends.codeinjections.getpagedimensions
+    return getpagedimensions()
+end
+
 function codeinjections.setupcanvas(specification)
     local paperheight = specification.paperheight
     local paperwidth  = specification.paperwidth
     local paperdouble = specification.doublesided
-    if paperheight then
-        texset('global','pageheight',paperheight)
-    end
-    if paperwidth then
-        texset('global','pagewidth',paperwidth)
-    end
+    --
+    paperwidth, paperheight = codeinjections.setpagedimensions(paperwidth,paperheight)
+    --
     pagespec    = specification.mode       or pagespec
     topoffset   = specification.topoffset  or 0
     leftoffset  = specification.leftoffset or 0
-    height      = specification.height     or texget("pageheight")
-    width       = specification.width      or texget("pagewidth")
+    height      = specification.height     or paperheight
+    width       = specification.width      or paperwidth
+    marked      = specification.print
     --
     copies      = specification.copies
     if copies and copies < 2 then
@@ -416,46 +463,69 @@ local function documentspecification()
             end
         end
     end
-    --
-    local layout = spec.layout
-    local mode   = spec.mode
-    local fit    = spec.fit
-    local fixed  = spec.fixed
-    local duplex = spec.duplex
-    local paper  = spec.paper
+    -- maybe interfaces.variables
+    local layout    = spec.layout
+    local mode      = spec.mode
+    local fit       = spec.fit
+    local fixed     = spec.fixed
+    local duplex    = spec.duplex
+    local paper     = spec.paper
+    local title     = spec.title
+    local direction = spec.direction
     if layout then
         addtocatalog("PageLayout",pdfconstant(layout))
     end
     if mode then
         addtocatalog("PageMode",pdfconstant(mode))
     end
-    if fit or fixed or duplex or copies or paper then
+    local prints = nil
+    if marked then
+        local pages     = structures.pages
+        local marked    = pages.allmarked(marked)
+        local nofmarked = marked and #marked or 0
+        if nofmarked > 0 then
+            -- the spec is wrong in saying that numbering starts at 1 which of course makes
+            -- sense as most real documents start with page 0 .. sigh
+            for i=1,#marked do marked[i] = marked[i] - 1 end
+            prints = pdfarray(flattened(pages.toranges(marked)))
+        end
+    end
+    if fit or fixed or duplex or copies or paper or prints or title or direction then
         addtocatalog("ViewerPreferences",pdfdictionary {
-            FitWindow         = fit    and true                or nil,
-            PrintScaling      = fixed  and pdfconstant("None") or nil,
-            Duplex            = duplex and pdfconstant(duplex) or nil,
-            NumCopies         = copies and copies              or nil,
-            PickTrayByPDFSize = paper  and true                or nil,
+            FitWindow         = fit       and true                   or nil,
+            PrintScaling      = fixed     and pdfconstant("None")    or nil,
+            Duplex            = duplex    and pdfconstant(duplex)    or nil,
+            NumCopies         = copies    and copies                 or nil,
+            PickTrayByPDFSize = paper     and true                   or nil,
+            PrintPageRange    = prints                               or nil,
+            DisplayDocTitle   = title     and true                   or nil,
+            Direction         = direction and pdfconstant(direction) or nil,
         })
     end
     addtoinfo   ("Trapped", pdfconstant("False")) -- '/Trapped' in /Info, 'Trapped' in XMP
-    addtocatalog("Version", pdfconstant(format("1.%s",pdf.getminorversion())))
+    addtocatalog("Version", pdfconstant(format("1.%s",pdfminorversion())))
+    addtocatalog("Lang",    pdfstring(tokens.getters.macro("currentmainlanguage")))
 end
 
 -- temp hack: the mediabox is not under our control and has a precision of 5 digits
 
 local factor  = number.dimenfactors.bp
-local f_value = formatters["%0.5F"]
+local f_value = formatters["%.6F"]
+
+directives.register("pdf.stripzeros",function()
+    local f_value = formatters["%.6N"]
+end)
 
 local function boxvalue(n) -- we could share them
     return pdfverbose(f_value(factor * n))
 end
 
 local function pagespecification()
+    local paperwidth, paperheight = codeinjections.getpagedimensions()
     local llx = leftoffset
-    local lly = texget("pageheight") + topoffset - height
+    local lly = paperheight + topoffset - height
     local urx = width - leftoffset
-    local ury = texget("pageheight") - topoffset
+    local ury = paperheight - topoffset
     -- boxes can be cached
     local function extrabox(WhatBox,offset,always)
         if offset ~= 0 or always then
@@ -467,10 +537,14 @@ local function pagespecification()
             })
         end
     end
-    extrabox("CropBox",cropoffset,true) -- mandate for rendering
-    extrabox("TrimBox",trimoffset,true) -- mandate for pdf/x
-    extrabox("BleedBox",bleedoffset)    -- optional
- -- extrabox("ArtBox",artoffset)        -- optional .. unclear what this is meant to do
+    if omitextraboxes then
+        -- only useful for testing / comparing
+    else
+        extrabox("CropBox",cropoffset,true) -- mandate for rendering
+        extrabox("TrimBox",trimoffset,true) -- mandate for pdf/x
+        extrabox("BleedBox",bleedoffset)    -- optional
+     -- extrabox("ArtBox",artoffset)        -- optional .. unclear what this is meant to do
+    end
 end
 
 lpdf.registerpagefinalizer(pagespecification,"page specification")
