@@ -6,10 +6,30 @@ if not modules then modules = { } end modules ['lpdf-wid'] = {
     license   = "see context related readme files"
 }
 
-local gmatch, gsub, find, lower, format = string.gmatch, string.gsub, string.find, string.lower, string.format
+-- It's about time to give up on media in pdf and admit that pdf lost it to html.
+-- First we had movies and sound, quite easy to deal with, but obsolete now. Then we
+-- had renditions but they turned out to be unreliable from the start and look
+-- obsolete too or at least they are bound to the (obsolete) flash technology for
+-- rendering. They were already complex constructs. Now we have rich media which
+-- instead of providing a robust future proof framework for general media types
+-- again seems to depend on viewers built in (yes, also kind of obsolete) flash
+-- technology, and we cannot expect this non-open technology to show up in open
+-- browsers. So, in the end we can best just use links to external resources to be
+-- future proof. Just look at the viewer preferences pane to see how fragile support
+-- is. Interestingly u3d support is kind of built in, while e.g. mp4 support relies
+-- on wrapping in swf. We used to stay ahead of the pack with support of the fancy
+-- pdf features but it backfires and is not worth the trouble. And yes, for control
+-- (even simple like starting and stopping videos) one has to revert to JavaScript,
+-- the other fragile bit. And, now that adobe quits flash in 2020 we're without any
+-- video anyway. Also, it won't play on all platforms and devices so let's wait for
+-- html5 media in pdf then.
+
+local tonumber, next = tonumber, next
+local gmatch, gsub, find, lower = string.gmatch, string.gsub, string.find, string.lower
 local stripstring = string.strip
 local settings_to_array = utilities.parsers.settings_to_array
 local settings_to_hash = utilities.parsers.settings_to_hash
+local sortedhash, sortedkeys = table.sortedhash, table.sortedkeys
 
 local report_media             = logs.reporter("backend","media")
 local report_attachment        = logs.reporter("backend","attachment")
@@ -29,11 +49,10 @@ local executers                = structures.references.executers
 local variables                = interfaces.variables
 
 local v_hidden                 = variables.hidden
-local v_normal                 = variables.normal
 local v_auto                   = variables.auto
 local v_embed                  = variables.embed
-local v_unknown                = variables.unknown
 local v_max                    = variables.max
+local v_yes                    = variables.yes
 
 local pdfconstant              = lpdf.constant
 local pdfdictionary            = lpdf.dictionary
@@ -42,7 +61,6 @@ local pdfreference             = lpdf.reference
 local pdfunicode               = lpdf.unicode
 local pdfstring                = lpdf.string
 local pdfboolean               = lpdf.boolean
-local pdfcolorspec             = lpdf.colorspec
 local pdfflushobject           = lpdf.flushobject
 local pdfflushstreamobject     = lpdf.flushstreamobject
 local pdfflushstreamfileobject = lpdf.flushstreamfileobject
@@ -123,6 +141,10 @@ local attachment_symbols = {
 attachment_symbols.PushPin = attachment_symbols.Pushpin
 attachment_symbols.Default = attachment_symbols.Pushpin
 
+function lpdf.attachmentsymbols()
+    return sortedkeys(comment_symbols)
+end
+
 local comment_symbols = {
     Comment      = pdfconstant("Comment"),
     Help         = pdfconstant("Help"),
@@ -136,10 +158,14 @@ local comment_symbols = {
 comment_symbols.NewParagraph = Newparagraph
 comment_symbols.Default      = Note
 
+function lpdf.commentsymbols()
+    return sortedkeys(comment_symbols)
+end
+
 local function analyzesymbol(symbol,collection)
     if not symbol or symbol == "" then
-        return collection.Default, nil
-    elseif collection[symbol] then
+        return collection and collection.Default, nil
+    elseif collection and collection[symbol] then
         return collection[symbol], nil
     else
         local setn, setr, setd
@@ -161,6 +187,17 @@ local function analyzesymbol(symbol,collection)
     end
 end
 
+local function analyzenormalsymbol(symbol)
+    local appearance = pdfdictionary {
+        N = registeredsymbol(symbol),
+    }
+    local appearanceref = pdfshareobjectreference(appearance)
+    return appearanceref
+end
+
+codeinjections.analyzesymbol       = analyzesymbol
+codeinjections.analyzenormalsymbol = analyzenormalsymbol
+
 local function analyzelayer(layer)
     -- todo:  (specification.layer ~= "" and pdfreference(specification.layer)) or nil, -- todo: ref to layer
 end
@@ -203,17 +240,19 @@ job.register('job.fileobjreferences.collected', tobesavedobjrefs, initializer)
 local function flushembeddedfiles()
     if next(filestreams) then
         local e = pdfarray()
-        for tag, reference in next, filestreams do
+        for tag, reference in sortedhash(filestreams) do
             if not reference then
                 report_attachment("unreferenced file, tag %a",tag)
             elseif referenced[tag] == "hidden" then
                 e[#e+1] = pdfstring(tag)
                 e[#e+1] = reference -- already a reference
             else
-                -- messy spec ... when annot not in named else twice in menu list acrobat
+         --     -- messy spec ... when annot not in named else twice in menu list acrobat
             end
         end
-        lpdf.addtonames("EmbeddedFiles",pdfreference(pdfflushobject(pdfdictionary{ Names = e })))
+        if #e > 0 then
+            lpdf.addtonames("EmbeddedFiles",pdfreference(pdfflushobject(pdfdictionary{ Names = e })))
+        end
     end
 end
 
@@ -228,9 +267,13 @@ function codeinjections.embedfile(specification)
     local keepdir  = specification.keepdir -- can change
     local usedname = specification.usedname
     local filetype = specification.filetype
+    local compress = specification.compress
     if filename == "" then
         filename = nil
     end
+    if compress == nil then
+        compress = true
+    end
     if data then
         local r = filestreams[hash]
         if r == false then
@@ -285,12 +328,13 @@ function codeinjections.embedfile(specification)
         specification.data = true -- signal that still data but already flushed
     else
         local foundname = specification.foundname or filename
-        f = pdfflushstreamfileobject(foundname,a)
+        f = pdfflushstreamfileobject(foundname,a,compress)
     end
     local d = pdfdictionary {
         Type = pdfconstant("Filespec"),
         F    = pdfstring(savename),
-        UF   = pdfstring(savename),
+     -- UF   = pdfstring(savename),
+        UF   = pdfunicode(savename),
         EF   = pdfdictionary { F = pdfreference(f) },
         Desc = title ~= "" and pdfunicode(title) or nil,
      -- AFRelationship = pdfconstant("Source"), -- some day maybe, not mandate
@@ -364,7 +408,7 @@ function nodeinjections.attachfile(specification)
             FS       = aref,
             Contents = pdfunicode(title),
             Name     = name,
-            NM       = pdfstring(format("attachment:%s",nofattachments)),
+            NM       = pdfstring("attachment:"..nofattachments),
             T        = author ~= "" and pdfunicode(author) or nil,
             Subj     = subtitle ~= "" and pdfunicode(subtitle) or nil,
             C        = analyzecolor(specification.colorvalue,specification.colormodel),
@@ -372,9 +416,13 @@ function nodeinjections.attachfile(specification)
             AP       = appearance,
             OC       = analyzelayer(specification.layer),
         }
-        local width, height, depth = specification.width or 0, specification.height or 0, specification.depth
-        local box = hpack_node(nodeinjections.annotation(width,height,depth,d()))
-        box.width, box.height, box.depth = width, height, depth
+        local width  = specification.width  or 0
+        local height = specification.height or 0
+        local depth  = specification.depth  or 0
+        local box    = hpack_node(nodeinjections.annotation(width,height,depth,d()))
+        box.width    = width
+        box.height   = height
+        box.depth    = depth
         return box
     end
 end
@@ -383,7 +431,8 @@ function codeinjections.attachmentid(filename) -- not used in context
     return filestreams[filename]
 end
 
-local nofcomments, usepopupcomments, stripleading = 0, false, true
+local nofcomments      = 0
+local usepopupcomments = false
 
 local defaultattributes = {
     ["xmlns"]           = "http://www.w3.org/1999/xhtml",
@@ -417,10 +466,12 @@ end
 
 function nodeinjections.comment(specification) -- brrr: seems to be done twice
     nofcomments = nofcomments + 1
-    local text = stripstring(specification.data or "")
-    if stripleading then
+    local text = specification.data or ""
+    if specification.space ~= v_yes then
+        text = stripstring(text)
         text = gsub(text,"[\n\r] *","\n")
     end
+    text = gsub(text,"\r","\n")
     local name, appearance = analyzesymbol(specification.symbol,comment_symbols)
     local tag      = specification.tag      or "" -- this is somewhat messy as recent
     local title    = specification.title    or "" -- versions of acrobat see the title
@@ -451,10 +502,12 @@ function nodeinjections.comment(specification) -- brrr: seems to be done twice
         CA        = analyzetransparency(specification.transparencyvalue),
         OC        = analyzelayer(specification.layer),
         Name      = name,
-        NM        = pdfstring(format("comment:%s",nofcomments)),
+        NM        = pdfstring("comment:"..nofcomments),
         AP        = appearance,
     }
-    local width, height, depth = specification.width or 0, specification.height or 0, specification.depth
+    local width  = specification.width  or 0
+    local height = specification.height or 0
+    local depth  = specification.depth  or 0
     local box
     if usepopupcomments then
         -- rather useless as we can hide/vide
@@ -472,7 +525,9 @@ function nodeinjections.comment(specification) -- brrr: seems to be done twice
     else
         box = hpack_node(nodeinjections.annotation(width,height,depth,d()))
     end
-    box.width, box.height, box.depth = width, height, depth -- redundant
+    box.width  = width  -- redundant
+    box.height = height -- redundant
+    box.depth  = depth  -- redundant
     return box
 end
 
@@ -572,7 +627,8 @@ local function insertrendering(specification)
     local option = settings_to_hash(specification.option)
     if not mf[label] then
         local filename = specification.filename
-        local isurl = find(filename,"://",1,true)
+        local isurl    = find(filename,"://",1,true)
+        local mimetype = specification.mimetype or specification.mime
      -- local start = pdfdictionary {
      --     Type = pdfconstant("MediaOffset"),
      --     S = pdfconstant("T"), -- time
@@ -608,13 +664,17 @@ local function insertrendering(specification)
         if isurl then
             descriptor.FS = pdfconstant("URL")
         elseif option[v_embed] then
-            descriptor.EF = codeinjections.embedfile { file = filename }
+            descriptor.EF = codeinjections.embedfile {
+                file     = filename,
+                mimetype = mimetype, -- yes or no
+                compress = false,
+            }
         end
         local clip = pdfdictionary {
             Type = pdfconstant("MediaClip"),
             S    = pdfconstant("MCD"),
             N    = label,
-            CT   = specification.mime,
+            CT   = mimetype,
             Alt  = pdfarray { "", "file not found" }, -- language id + message
             D    = pdfreference(pdfflushobject(descriptor)),
          -- P    = pdfreference(pdfflushobject(parameters)),
