@@ -19,7 +19,7 @@ texconfig.shell_escape = 't'
 luatex       = luatex or { }
 local luatex = luatex
 
-texconfig.error_line      =     79 -- frozen
+texconfig.error_line      =     79 -- frozen / large values can crash
 texconfig.expand_depth    =  10000
 texconfig.half_error_line =     50 -- frozen
 texconfig.hash_extra      = 100000
@@ -30,6 +30,8 @@ texconfig.nest_size       =   1000
 texconfig.param_size      =  25000
 texconfig.save_size       = 100000
 texconfig.stack_size      =  10000
+texconfig.function_size   =  32768
+texconfig.properties_size =  65536
 
 local stub = [[
 
@@ -67,7 +69,7 @@ function texconfig.init()
             "gzip",  "zip", "zlib", "lfs", "ltn12", "mime", "socket", "md5", "fio", "unicode", "utf",
         },
         extratex = {
-            "epdf", "fontloader", "kpse", "mplib",
+            "epdf", "kpse", "mplib", -- "fontloader",
         },
         obsolete = {
             "fontloader", -- can be filled by luat-log
@@ -119,16 +121,21 @@ function texconfig.init()
 
     -- shortcut and helper
 
-    local bytecode = lua.bytecode
+    local setbytecode = lua.setbytecode
+    local getbytecode = lua.getbytecode
 
     local function init(start)
         local i = start
         local t = os.clock()
-        while bytecode[i] do
-            bytecode[i]() ;
-            bytecode[i] = nil ;
-            i = i + 1
-         -- collectgarbage('step')
+        while true do
+            local b = getbytecode(i)
+            if b then
+                b() ;
+                setbytecode(i,nil) ;
+                i = i + 1
+            else
+                break
+            end
         end
         return i - start, os.clock() - t
     end
@@ -149,6 +156,8 @@ function texconfig.init()
 
 end
 
+CONTEXTLMTXMODE = %s
+
 -- we provide a qualified path
 
 callback.register('find_format_file',function(name)
@@ -171,6 +180,8 @@ local variablenames = {
     param_size      = true,
     save_size       = true,
     stack_size      = true,
+    function_size   = true,
+    properties_size = true,
 }
 
 local function makestub()
@@ -207,7 +218,7 @@ local function makestub()
             t[#t+1] = format("texconfig.%s=%s",v,tv)
         end
     end
-    io.savedata(name,format("%s\n\n%s",concat(t,"\n"),format(stub,firsttable)))
+    io.savedata(name,format("%s\n\n%s",concat(t,"\n"),format(stub,firsttable,tostring(CONTEXTLMTXMODE) or 0)))
     logs.newline()
 end
 
