@@ -15,7 +15,6 @@ local type, next = type, next
 local insert, remove, unpack, concat, merge = table.insert, table.remove, table.unpack, table.concat, table.merge
 local find, topattern, format = string.find, string.topattern, string.format
 local lpegmatch, P, S, C, Cc = lpeg.match, lpeg.P, lpeg.S, lpeg.C, lpeg.Cc
-local texattribute = tex.attribute
 local allocate = utilities.storage.allocate
 local settings_to_hash = utilities.parsers.settings_to_hash
 local setmetatableindex = table.setmetatableindex
@@ -24,34 +23,44 @@ local trace_tags = false  trackers.register("structures.tags", function(v) trace
 
 local report_tags = logs.reporter("structure","tags")
 
-local attributes     = attributes
-local structures     = structures
-local implement      = interfaces.implement
+local attributes      = attributes
+local structures      = structures
+local implement       = interfaces.implement
 
-local a_tagged       = attributes.private('tagged')
+local a_tagged        = attributes.private('tagged')
 
-local unsetvalue     = attributes.unsetvalue
-local codeinjections = backends.codeinjections
+local unsetvalue      = attributes.unsetvalue
+local codeinjections  = backends.codeinjections
 
-local taglist        = allocate() -- access by attribute
-local specifications = allocate() -- access by fulltag
-local labels         = allocate()
-local stack          = { }
-local chain          = { }
-local ids            = { }
-local enabled        = false
-local tagcontext     = { }
-local tagpatterns    = { }
-local lasttags       = { }
-local stacksize      = 0
-local metadata       = nil -- applied to the next element
-local documentdata   = { }
+local texgetattribute = tex.getattribute
+local texsetattribute = tex.setattribute
 
-local tags           = structures.tags
-tags.taglist         = taglist -- can best be hidden
-tags.labels          = labels
-tags.patterns        = tagpatterns
-tags.specifications  = specifications
+local taglist         = allocate() -- access by attribute
+local specifications  = allocate() -- access by fulltag
+local labels          = allocate()
+local stack           = { }
+local chain           = { }
+local ids             = { }
+local enabled         = false
+local tagcontext      = { }
+local tagpatterns     = { }
+local lasttags        = { }
+local stacksize       = 0
+local metadata        = nil -- applied to the next element
+local documentdata    = { }
+local extradata       = false
+
+local tags            = structures.tags
+tags.taglist          = taglist -- can best be hidden
+tags.labels           = labels
+tags.patterns         = tagpatterns
+tags.specifications   = specifications
+
+function tags.current()
+    if stacksize > 0 then
+        return stack[stacksize] -- maybe copy or proxy
+    end
+end
 
 -- Tags are internally stored as:
 --
@@ -60,7 +69,7 @@ tags.specifications  = specifications
 local p_splitter     = C((1-S(">"))^1) * P(">") * C(P(1)^1)
 tagpatterns.splitter = p_splitter
 
-local properties     = allocate {
+local properties     = allocate { -- todo: more "record = true" to improve formatting
 
     document              = { pdf = "Div",        nature = "display" },
 
@@ -71,6 +80,7 @@ local properties     = allocate {
     highlight             = { pdf = "Span",       nature = "inline"  },
 
     section               = { pdf = "Sect",       nature = "display" },
+    sectioncaption        = { pdf = "Div",        nature = "display", record = true },
     sectiontitle          = { pdf = "H",          nature = "mixed"   },
     sectionnumber         = { pdf = "H",          nature = "mixed"   },
     sectioncontent        = { pdf = "Div",        nature = "display" },
@@ -125,6 +135,7 @@ local properties     = allocate {
     listcontent           = { pdf = "P",          nature = "mixed"   },
     listdata              = { pdf = "P",          nature = "mixed"   },
     listpage              = { pdf = "Reference",  nature = "mixed"   },
+    listtext              = { pdf = "Span",       nature = "inline"  },
 
     delimitedblock        = { pdf = "BlockQuote", nature = "display" },
     delimited             = { pdf = "Quote",      nature = "inline"  },
@@ -156,9 +167,11 @@ local properties     = allocate {
     subformula            = { pdf = "Div",        nature = "display" },
 
     link                  = { pdf = "Link",       nature = "inline"  },
+    reference             = { pdf = "Span",       nature = "inline"  },
 
     margintextblock       = { pdf = "Span",       nature = "inline"  },
     margintext            = { pdf = "Span",       nature = "inline"  },
+    marginanchor          = { pdf = "Span",       nature = "inline"  },
 
     math                  = { pdf = "Div",        nature = "inline"  }, -- no display
     mn                    = { pdf = "Span",       nature = "mixed"   },
@@ -203,6 +216,14 @@ local properties     = allocate {
     combinationpair       = { pdf = "Span",       nature = "display" },
     combinationcontent    = { pdf = "Span",       nature = "mixed"   },
     combinationcaption    = { pdf = "Span",       nature = "mixed"   },
+
+    publications          = { pdf = "Div",        nature = "display" },
+    publication           = { pdf = "Div",        nature = "mixed"   },
+    pubfld                = { pdf = "Span",       nature = "inline"  },
+
+    block                 = { pdf = "Div",        nature = "display"  },
+    userdata              = { pdf = "Div",        nature = "display"  },
+
 }
 
 tags.properties = properties
@@ -214,7 +235,7 @@ local patterns = setmetatableindex(function(t,tag)
 end)
 
 function tags.locatedtag(tag)
-    local attribute = texattribute[a_tagged]
+    local attribute = texgetattribute(a_tagged)
     if attribute >= 0 then
         local specification = taglist[attribute]
         if specification then
@@ -234,7 +255,7 @@ function tags.locatedtag(tag)
 end
 
 function structures.atlocation(str)
-    local specification = taglist[texattribute[a_tagged]]
+    local specification = taglist[texgetattribute(a_tagged)]
     if specification then
         if list then
             local taglist = specification.taglist
@@ -282,6 +303,24 @@ function tags.registermetadata(data)
     end
 end
 
+function tags.getmetadata()
+    return documentdata or { }
+end
+
+function tags.registerextradata(name,serializer)
+    if type(serializer) == "function" then
+        if extradata then
+            extradata[name] = serializer
+        else
+            extradata = { [name] = serializer }
+        end
+    end
+end
+
+function tags.getextradata()
+    return extradata
+end
+
 function tags.start(tag,specification)
     if not enabled then
         codeinjections.enabletags()
@@ -344,7 +383,7 @@ function tags.start(tag,specification)
         specification.metadata = documentdata
     end
     --
-    texattribute[a_tagged] = attribute
+    texsetattribute(a_tagged,attribute)
     return attribute
 end
 
@@ -359,7 +398,7 @@ function tags.restart(attribute)
         taglist[attribute] = { taglist = { unpack(chain,1,stacksize) } }
     end
     stack[stacksize] = attribute
-    texattribute[a_tagged] = attribute
+    texsetattribute(a_tagged,attribute)
     return attribute
 end
 
@@ -369,12 +408,12 @@ function tags.stop()
     end
     local t = stack[stacksize]
     if not t then
-     -- if trace_tags then
+        if trace_tags then
             report_tags("ignoring end tag, previous chain: %s",stacksize > 0 and concat(chain," ",1,stacksize) or "none")
-     -- end
+        end
         t = unsetvalue
     end
-    texattribute[a_tagged] = t
+    texsetattribute(a_tagged,t)
     return t
 end
 
@@ -459,7 +498,7 @@ local starttag = tags.start
 implement {
     name      = "starttag",
     actions   = starttag,
-    arguments = { "string" }
+    arguments = "string",
 }
 
 implement {
@@ -471,26 +510,25 @@ implement {
     name      = "starttag_u",
     scope     = "private",
     actions   = function(tag,userdata) starttag(tag,{ userdata = userdata }) end,
-    arguments = { "string", "string" }
+    arguments = "2 strings",
 }
 
 implement {
     name      = "starttag_d",
     scope     = "private",
     actions   = function(tag,detail) starttag(tag,{ detail = detail }) end,
-    arguments = { "string", "string" }
+    arguments = "2 strings",
 }
 
 implement {
     name      = "starttag_c",
     scope     = "private",
     actions   = function(tag,detail,parents) starttag(tag,{ detail = detail, parents = parents }) end,
-    arguments = { "string", "string", "string" }
+    arguments = "3 strings",
 }
 
-implement { name = "settagaspect",     actions = tags.setaspect,   arguments = { "string", "string" } }
-
-implement { name = "settagproperty",   actions = tags.setproperty, arguments = { "string", "string", "string" } }
+implement { name = "settagaspect",     actions = tags.setaspect,   arguments = "2 strings" }
+implement { name = "settagproperty",   actions = tags.setproperty, arguments = "3 strings" }
 implement { name = "settagproperty_b", actions = tags.setproperty, arguments = { "string", "'backend'", "string" }, scope = "private" }
 implement { name = "settagproperty_n", actions = tags.setproperty, arguments = { "string", "'nature'",  "string" }, scope = "private" }
 
@@ -500,7 +538,7 @@ implement {
     name      = "setelementuserproperties",
     scope     = "private",
     actions   = tags.setuserproperties,
-    arguments = { "string", "string" }
+    arguments = "2 strings",
 }
 
 implement {
@@ -512,5 +550,5 @@ implement {
 implement {
     name      = "settaggedmetadata",
     actions   = tags.registermetadata,
-    arguments = "string"
+    arguments = "string",
 }
