@@ -22,6 +22,8 @@ local tokenbits      = tokens.bits
 
 local scanstring     = scanners.string
 local scanargument   = scanners.argument
+local scanverbatim   = scanners.verbatim
+local scantokenlist  = scanners.tokenlist
 local scaninteger    = scanners.integer
 local scannumber     = scanners.number
 local scankeyword    = scanners.keyword
@@ -106,7 +108,40 @@ local function scanconditional()
     return nil
 end
 
+local function scantable(t,data)
+    if not data then
+        data = { }
+    end
+    local wrapped = scanopen()
+    while true do
+        local key = scanword()
+        if key then
+            local get = t[key]
+            if get then
+                data[key] = get()
+            else
+                -- catch all we can get
+            end
+        else
+            break
+        end
+    end
+    if wrapped then
+        scanclose()
+    end
+    return data
+end
+
+function tokens.constant(s)
+    if type(s) == "string" then
+        return "'" .. s .. "'"
+    else
+        return s
+    end
+end
+
 scanners.list        = scanlist
+scanners.table       = scantable
 scanners.conditional = scanconditional
 
 local shortcuts = {
@@ -116,8 +151,12 @@ local shortcuts = {
     close           = close,
     scanners        = scanners,
     scanstring      = scanstring,
+    scanargument    = scanargument,
+    scanverbatim    = scanverbatim,
+    scantokenlist   = scantokenlist,
     scaninteger     = scaninteger,
     scannumber      = scannumber,
+    scantable       = scantable,
     scankeyword     = scankeyword,
     scankeywordcs   = scankeywordcs,
     scanword        = scanword,
@@ -254,6 +293,8 @@ local presets = {
     ["8 strings"] = { "string", "string", "string", "string", "string", "string", "string", "string" },
 }
 
+tokens.presets = presets
+
 function tokens.compile(specification)
     local f = { }
     local n = 0
@@ -319,7 +360,7 @@ function tokens.compile(specification)
             return c
         end
     end
-    local p = t and presets[t]
+    local p = t and presets[t] -- already done in implement
     if p then
         t = p
     end
@@ -460,35 +501,3 @@ end
 -- }
 --
 -- os.exit()
-
-function tokens.scantable(t,data)
-    if not data then
-        data = { }
-    end
-    local wrapped = scanopen()
-    while true do
-        local key = scanword()
-        if key then
-            local get = t[key]
-            if get then
-                data[key] = get()
-            else
-                -- catch all we can get
-            end
-        else
-            break
-        end
-    end
-    if wrapped then
-        scanclose()
-    end
-    return data
-end
-
-function tokens.constant(s)
-    if type(s) == "string" then
-        return "'" .. s .. "'"
-    else
-        return s
-    end
-end
