@@ -30,7 +30,6 @@ local context            = context
 local implement          = interfaces.implement
 
 local nuts               = nodes.nuts
-local tonut              = nuts.tonut
 local tonode             = nuts.tonode
 
 local getnext            = nuts.getnext
@@ -38,8 +37,7 @@ local getprev            = nuts.getprev
 local getboth            = nuts.getboth
 local setboth            = nuts.setboth
 local getid              = nuts.getid
-local getfield           = nuts.getfield
-local setfield           = nuts.setfield
+local getwidth           = nuts.getwidth
 local getlist            = nuts.getlist
 local setlist            = nuts.setlist
 local getattr            = nuts.getattr
@@ -54,14 +52,21 @@ local nodecodes          = nodes.nodecodes
 local glyph_code         = nodecodes.glyph
 local disc_code          = nodecodes.disc
 local kern_code          = nodecodes.kern
+local glue_code          = nodecodes.glue
+
+local spaceskip_code     = nodes.gluecodes.spaceskip
+
+local nextglyph          = nuts.traversers.glyph
+local nextdisc           = nuts.traversers.disc
 
-local traverse_id        = nuts.traverse_id
 local flush_node_list    = nuts.flush_list
 local flush_node         = nuts.flush_node
 local copy_node_list     = nuts.copy_list
+local insert_node_before = nuts.insert_before
 local insert_node_after  = nuts.insert_after
 local remove_node        = nuts.remove
-local list_dimensions    = nuts.dimensions
+local getdimensions      = nuts.dimensions
+local hpack_node_list    = nuts.hpack
 
 local nodepool           = nuts.pool
 local newpenalty         = nodepool.penalty
@@ -115,7 +120,7 @@ implement {
 }
 
 actions[v_line] = function(head,setting)
- -- local attribute = fonts.specifiers.contextnumber(setting.feature) -- was experimental
+ -- local attribute  = fonts.specifiers.contextnumber(setting.feature) -- was experimental
     local dynamic    = setting.dynamic
     local font       = setting.font
     local noflines   = setting.n or 1
@@ -131,7 +136,7 @@ actions[v_line] = function(head,setting)
     local linebreaks = { }
 
     local function set(head)
-        for g in traverse_id(glyph_code,head) do
+        for g in nextglyph, head do
             if dynamic > 0 then
                 setattr(g,0,dynamic)
             end
@@ -141,7 +146,7 @@ actions[v_line] = function(head,setting)
 
     set(temp)
 
-    for g in traverse_id(disc_code,temp) do
+    for g in nextdisc, temp do
         local pre, post, replace = getdisc(g)
         if pre then
             set(pre)
@@ -166,11 +171,24 @@ actions[v_line] = function(head,setting)
             hsize = hsize - hangindent
         end
 
+        local function list_dimensions(list,start)
+            local temp = copy_node_list(list,start)
+            temp = nodes.handlers.characters(temp)
+            temp = nodes.injections.handler(temp)
+         -- temp = typesetters.fontkerns.handler(temp) -- maybe when enabled
+         --        nodes.handlers.protectglyphs(temp)  -- not needed as we discard
+         -- temp = typesetters.spacings.handler(temp)  -- maybe when enabled
+         -- temp = typesetters.kerns.handler(temp)     -- maybe when enabled
+            local width = getdimensions(temp)
+            return width
+        end
+
         local function try(extra)
             local width = list_dimensions(list,start)
             if extra then
                 width = width + list_dimensions(extra)
             end
+         -- report_firstlines("line length: %p, progression: %p, text: %s",hsize,width,nodes.listtoutf(list,nil,nil,start))
             if width > hsize then
                 list = prev
                 return true
@@ -184,16 +202,20 @@ actions[v_line] = function(head,setting)
         while start do
             local id = getid(start)
             if id == glyph_code then
-                n = n + 1
+                -- go on
             elseif id == disc_code then
                 -- this could be an option
                 n = n + 1
-                if try(getfield(start,"pre")) then
+                local pre, post, replace = getdisc(start)
+                if pre and try(pre) then
+                    break
+                elseif replace and try(replace) then
                     break
                 end
             elseif id == kern_code then -- todo: fontkern
                 -- this could be an option
-            elseif n > 0 then
+            elseif id == glue_code then
+                n = n + 1
                 if try() then
                     break
                 end
@@ -204,6 +226,9 @@ actions[v_line] = function(head,setting)
             linebreaks[i] = n
         end
     end
+
+    flush_node_list(temp)
+
     local start = head
     local n     = 0
 
@@ -225,8 +250,8 @@ actions[v_line] = function(head,setting)
         local linebreak = linebreaks[i]
         while start and n < nofchars do
             local id = getid(start)
+            local ok = false
             if id == glyph_code then
-                n = n + 1
                 update(start)
             elseif id == disc_code then
                 n = n + 1
@@ -235,7 +260,7 @@ actions[v_line] = function(head,setting)
                 if linebreak == n then
                     local p, n = getboth(start)
                     if pre then
-                        for current in traverse_id(glyph_code,pre) do
+                        for current in nextglyph, pre do
                             update(current)
                         end
                         setlink(pretail,n)
@@ -255,7 +280,7 @@ actions[v_line] = function(head,setting)
                 else
                     local p, n = getboth(start)
                     if replace then
-                        for current in traverse_id(glyph_code,replace) do
+                        for current in nextglyph, replace do
                             update(current)
                         end
                         setlink(replacetail,n)
@@ -269,21 +294,31 @@ actions[v_line] = function(head,setting)
                 end
                 setdisc(disc,pre,post,replace)
                 flush_node(disc)
-            end
-            if linebreak == n then
-                if trace_firstlines then
-                    head, start = insert_node_after(head,start,newpenalty(10000)) -- nobreak
-                    head, start = insert_node_after(head,start,newkern(-65536))
-                    head, start = insert_node_after(head,start,tracerrule(65536,4*65536,2*65536,"darkblue"))
+            elseif id == glue_code then
+                n = n + 1
+                if linebreak ~= n then
+                    head = insert_node_before(head,start,newpenalty(10000)) -- nobreak
                 end
-                head, start = insert_node_after(head,start,newpenalty(-10000)) -- break
+            end
+            local next = getnext(start)
+            if linebreak == n then
+                if start ~= head then
+                    local where = id == glue_code and getprev(start) or start
+                    if trace_firstlines then
+                        head, where = insert_node_after(head,where,newpenalty(10000)) -- nobreak
+                        head, where = insert_node_after(head,where,newkern(-65536))
+                        head, where = insert_node_after(head,where,tracerrule(65536,4*65536,2*65536,"darkblue"))
+                    end
+                    head, where = insert_node_after(head,where,newpenalty(-10000)) -- break
+                end
+                start = next
                 break
             end
-            start = getnext(start)
+            start = next
         end
     end
-    flush_node_list(temp)
-    return head, true
+
+    return head
 end
 
 actions[v_word] = function(head,setting)
@@ -328,13 +363,12 @@ actions[v_word] = function(head,setting)
         end
         start = getnext(start)
     end
-    return head, true
+    return head
 end
 
 actions[v_default] = actions[v_line]
 
 function firstlines.handler(head)
-    head = tonut(head)
     local start = head
     local attr  = nil
     while start do
@@ -357,11 +391,10 @@ function firstlines.handler(head)
             if trace_firstlines then
                 report_firstlines("processing firstlines, alternative %a",alternative)
             end
-            local head, done = action(head,settings)
-            return tonode(head), done
+            return action(head,settings)
         end
     end
-    return tonode(head), false
+    return head
 end
 
 -- goodie
@@ -370,7 +403,7 @@ local function applytofirstcharacter(box,what)
     local tbox = getbox(box) -- assumes hlist
     local list = getlist(tbox)
     local done = nil
-    for n in traverse_id(glyph_code,list) do
+    for n in nextglyph, list do
         list = remove_node(list,n)
         done = n
         break
