@@ -6,6 +6,8 @@ if not modules then modules = { } end modules ['typo-sus'] = {
     license   = "see context related readme files"
 }
 
+local next = next
+
 local punctuation = {
     po = true,
 }
@@ -38,13 +40,10 @@ local hlist_code      = nodecodes.hlist
 local vlist_code      = nodecodes.vlist
 
 local nuts            = nodes.nuts
-local tonut           = nodes.tonut
-local tonode          = nodes.tonode
 
 local getid           = nuts.getid
 local getprev         = nuts.getprev
 local getnext         = nuts.getnext
-local getfield        = nuts.getfield
 local getattr         = nuts.getattr
 local getfont         = nuts.getfont
 local getlist         = nuts.getlist
@@ -163,7 +162,6 @@ local colors = {
 local found = 0
 
 function typesetters.marksuspects(head)
-    local head     = tonut(head)
     local current  = head
     local lastdone = nil
     while current do
@@ -253,7 +251,7 @@ function typesetters.marksuspects(head)
             current = getnext(current)
         end
     end
-    return tonode(head), found > 0
+    return head
 end
 
 local function showsuspects(head)
@@ -288,9 +286,9 @@ end
 
 function typesetters.showsuspects(head)
     if found > 0 then
-        return tonode(showsuspects(tonut(head))), true
+        return showsuspects(head)
     else
-        return head, false
+        return head
     end
 end
 
